@@ -1,6 +1,6 @@
 /* pdp11_defs.h: PDP-11 simulator definitions
 
-   Copyright (c) 1993-2005, Robert M Supnik
+   Copyright (c) 1993-2006, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -26,6 +26,10 @@
    The author gratefully acknowledges the help of Max Burnet, Megan Gentry,
    and John Wilson in resolving questions about the PDP-11
 
+   06-Jul-06    RMS     Added multiple KL11/DL11 support
+   26-Jun-06    RMS     Added RF11 support
+   24-May-06    RMS     Added 11/44 DR support (from CIS diagnostic)
+   17-May-06    RMS     Added CR11/CD11 support (from John Dundas)
    30-Sep-04    RMS     Added Massbus support
                         Removed Map_Addr prototype
                         Removed map argument from Unibus routines
@@ -137,7 +141,7 @@
 
 #define CPUT_F          (CPUT_23|CPUT_23P|CPUT_24)      /* all F11's */
 #define CPUT_J          (CPUT_53|CPUT_73|CPUT_73B| \
-                     CPUT_83|CPUT_84|CPUT_93|CPUT_94)
+                         CPUT_83|CPUT_84|CPUT_93|CPUT_94)
 #define CPUT_JB         (CPUT_73B|CPUT_83|CPUT_84)      /* KDJ11B */
 #define CPUT_JE         (CPUT_93|CPUT_94)               /* KDJ11E */
 #define CPUT_JU         (CPUT_84|CPUT_94)               /* KTJ11B UBA */
@@ -162,7 +166,7 @@
 
 /* Feature sets
 
-   SDSD                 source addr, source fetch, dest addr, dest fetch
+   SDSD                 source addr, dest addr, source fetch, dest fetch
    SR                   switch register
    DR                   display register
    RTT                  RTT instruction
@@ -195,7 +199,7 @@
 #define HAS_SR          (CPUT_04|CPUT_05|CPUT_20|CPUT_34|CPUT_40| \
                          CPUT_44|CPUT_45|CPUT_60|CPUT_70)
 #define HAS_DR          (CPUT_04|CPUT_05|CPUT_20|CPUT_24|CPUT_34| \
-                         CPUT_40|CPUT_45|CPUT_60|CPUT_70)
+                         CPUT_40|CPUT_44|CPUT_45|CPUT_60|CPUT_70)
 #define HAS_RTT         (CPUT_03|CPUT_04|CPUT_F|CPUT_34|CPUT_40| \
                          CPUT_44|CPUT_45|CPUT_60|CPUT_70|CPUT_J|CPUT_T)
 #define HAS_SXS         (CPUT_03|CPUT_F|CPUT_34|CPUT_40|CPUT_44| \
@@ -463,6 +467,7 @@ typedef struct {
 #define DZ_MUXES        4                               /* max # of DZ muxes */
 #define DZ_LINES        8                               /* lines per DZ mux */
 #define VH_MUXES        4                               /* max # of VH muxes */
+#define TTX_LINES       16                              /* max # of KL11/DL11's */
 #define MT_MAXFR        (1 << 16)                       /* magtape max rec */
 #define AUTO_LNT        34                              /* autoconfig ranks */
 #define DIB_MAX         100                             /* max DIBs */
@@ -546,8 +551,12 @@ typedef struct pdp_dib DIB;
 #define IOLN_TQ         004
 #define IOBA_XU         (IOPAGEBASE + 014510)           /* DEUNA/DELUA */
 #define IOLN_XU         010
+#define IOBA_TTIX       (IOPAGEBASE + 016500)           /* extra KL11/DL11 */
+#define IOLN_TTIX       (TTX_LINES * 010)
 #define IOBA_RP         (IOPAGEBASE + 016700)           /* RP/RM */
 #define IOLN_RP         054
+#define IOBA_CR         (IOPAGEBASE + 017160)           /* CD/CR/CM */
+#define IOLN_CR         010
 #define IOBA_RX         (IOPAGEBASE + 017170)           /* RX11 */
 #define IOLN_RX         004
 #define IOBA_RY         (IOPAGEBASE + 017170)           /* RY11 */
@@ -558,6 +567,8 @@ typedef struct pdp_dib DIB;
 #define IOLN_RK         020
 #define IOBA_HK         (IOPAGEBASE + 017440)           /* RK611 */
 #define IOLN_HK         040
+#define IOBA_RF         (IOPAGEBASE + 017460)           /* RF11 */
+#define IOLN_RF         020
 #define IOBA_LPT        (IOPAGEBASE + 017514)           /* LP11 */
 #define IOLN_LPT        004
 #define IOBA_CTL        (IOPAGEBASE + 017520)           /* board ctrl */
@@ -619,7 +630,8 @@ typedef struct pdp_dib DIB;
 #define INT_V_XQ        12
 #define INT_V_XU        13
 #define INT_V_TU        14
-#define INT_V_PIR5      15
+#define INT_V_RF        15
+#define INT_V_PIR5      16
 
 #define INT_V_TTI       0                               /* BR4 */
 #define INT_V_TTO       1
@@ -628,7 +640,10 @@ typedef struct pdp_dib DIB;
 #define INT_V_LPT       4
 #define INT_V_VHRX      5
 #define INT_V_VHTX      6  
-#define INT_V_PIR4      7
+#define INT_V_CR        7
+#define INT_V_TTIX      8
+#define INT_V_TTOX      9
+#define INT_V_PIR4      10
 
 #define INT_V_PIR3      0                               /* BR3 */
 #define INT_V_PIR2      0                               /* BR2 */
@@ -654,6 +669,7 @@ typedef struct pdp_dib DIB;
 #define INT_XQ          (1u << INT_V_XQ)
 #define INT_XU          (1u << INT_V_XU)
 #define INT_TU          (1u << INT_V_TU)
+#define INT_RF          (1u << INT_V_RF)
 #define INT_PIR5        (1u << INT_V_PIR5)
 #define INT_PTR         (1u << INT_V_PTR)
 #define INT_PTP         (1u << INT_V_PTP)
@@ -662,6 +678,9 @@ typedef struct pdp_dib DIB;
 #define INT_LPT         (1u << INT_V_LPT)
 #define INT_VHRX        (1u << INT_V_VHRX)
 #define INT_VHTX        (1u << INT_V_VHTX)
+#define INT_CR          (1u << INT_V_CR)
+#define INT_TTIX        (1u << INT_V_TTIX)
+#define INT_TTOX        (1u << INT_V_TTOX)
 #define INT_PIR4        (1u << INT_V_PIR4)
 #define INT_PIR3        (1u << INT_V_PIR3)
 #define INT_PIR2        (1u << INT_V_PIR2)
@@ -685,6 +704,7 @@ typedef struct pdp_dib DIB;
 #define IPL_XQ          5
 #define IPL_XU          5
 #define IPL_TU          5
+#define IPL_RF          5
 #define IPL_PTR         4
 #define IPL_PTP         4
 #define IPL_TTI         4
@@ -692,6 +712,9 @@ typedef struct pdp_dib DIB;
 #define IPL_LPT         4
 #define IPL_VHRX        4
 #define IPL_VHTX        4
+#define IPL_CR          4
+#define IPL_TTIX        4
+#define IPL_TTOX        4
 
 #define IPL_PIR7        7
 #define IPL_PIR6        6
@@ -716,16 +739,20 @@ typedef struct pdp_dib DIB;
 #define VEC_RQ          0154
 #define VEC_RL          0160
 #define VEC_LPT         0200
+#define VEC_RF          0204
 #define VEC_HK          0210
 #define VEC_RK          0220
 #define VEC_DTA         0214
 #define VEC_TM          0224
 #define VEC_TS          0224
 #define VEC_TU          0224
+#define VEC_CR          0230
 #define VEC_RP          0254
 #define VEC_TQ          0260
 #define VEC_RX          0264
 #define VEC_RY          0264
+#define VEC_TTIX        0300
+#define VEC_TTOX        0304
 #define VEC_DZRX        0300
 #define VEC_DZTX        0304
 #define VEC_VHRX        0310
@@ -780,6 +807,7 @@ int32 mba_get_csr (uint32 mbus);
 void mba_upd_ata (uint32 mbus, uint32 val);
 void mba_set_exc (uint32 mbus);
 void mba_set_don (uint32 mbus);
+void mba_set_enbdis (uint32 mb, t_bool dis);
 t_stat mba_show_num (FILE *st, UNIT *uptr, int32 val, void *desc);
 
 #endif
