@@ -1,6 +1,6 @@
 /* pdp11_tc.c: PDP-11 DECtape simulator
 
-   Copyright (c) 1993-2005, Robert M Supnik
+   Copyright (c) 1993-2006, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -25,6 +25,8 @@
 
    tc           TC11/TU56 DECtape
 
+   23-Jun-06	RMS     Fixed switch conflict in ATTACH
+   10-Feb-06    RMS     READ sets extended data bits in TCST (found by Alan Frisbie)
    16-Aug-05    RMS     Fixed C++ declaration and cast problems
    07-Jul-05    RMS     Removed extraneous externs
    30-Sep-04    RMS     Revised Unibus interface
@@ -866,6 +868,7 @@ switch (fnc) {                                          /* at speed, check fnc *
             ma = (CSR_GETMEX (tccm) << 16) | tcba;      /* form 18b addr */
             ba = (blk * DTU_BSIZE (uptr)) + wrd;        /* buffer ptr */
             tcdt = wbuf = fbuf[ba] & DMASK;             /* read word */
+            tcst = (tcst & ~STA_M_XD) | ((fbuf[ma] >> 16) & STA_M_XD);
             if (Map_WriteW (ma, 2, &wbuf)) {            /* store, nxm? */
                 dt_seterr (uptr, STA_NXM);
                 break;
@@ -1177,11 +1180,11 @@ r = attach_unit (uptr, cptr);                           /* attach */
 if (r != SCPE_OK) return r;                             /* fail? */
 if ((sim_switches & SIM_SW_REST) == 0) {                /* not from rest? */
     uptr->flags = (uptr->flags | UNIT_11FMT) & ~UNIT_8FMT;      /* default 16b */
-    if (sim_switches & SWMASK ('R'))                    /* att 12b? */
+    if (sim_switches & SWMASK ('T'))                    /* att 12b? */
         uptr->flags = (uptr->flags | UNIT_8FMT) & ~UNIT_11FMT;
-    else if (sim_switches & SWMASK ('T'))               /* att 18b? */
+    else if (sim_switches & SWMASK ('F'))               /* att 18b? */
         uptr->flags = uptr->flags & ~(UNIT_8FMT | UNIT_11FMT);
-    else if (!(sim_switches & SWMASK ('S')) &&          /* autosize? */
+    else if (!(sim_switches & SWMASK ('A')) &&          /* autosize? */
         ((sz = sim_fsize (uptr->fileref)) > D16_FILSIZ)) {
         if (sz <= D8_FILSIZ)
         uptr->flags = (uptr->flags | UNIT_8FMT) & ~UNIT_11FMT;
