@@ -1,6 +1,6 @@
 /* vax_syslist.c: VAX device list
 
-   Copyright (c) 1998-2005, Robert M Supnik
+   Copyright (c) 1998-2006, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -23,7 +23,8 @@
    used in advertising or otherwise to promote the sale, use or other dealings
    in this Software without prior written authorization from Robert M Supnik.
 
-   01-Oct-2004  RMS     Cloned from vax_sys.c
+   17-May-06    RMS     Added CR11/CD11 support (from John Dundas)
+   01-Oct-04    RMS     Cloned from vax_sys.c
 */
 
 #include "vax_defs.h"
@@ -40,6 +41,7 @@ extern DEVICE clk_dev;
 extern DEVICE tmr_dev;
 extern DEVICE tti_dev, tto_dev;
 extern DEVICE fl_dev;
+extern DEVICE cr_dev;
 extern DEVICE lpt_dev;
 extern DEVICE rq_dev, rqb_dev, rqc_dev, rqd_dev;
 extern DEVICE rl_dev;
@@ -72,6 +74,7 @@ DEVICE *sim_devices[] = {
     &tto_dev,
     &fl_dev,
     &dz_dev,
+    &cr_dev,
     &lpt_dev,
     &rp_dev,
     &rl_dev,
