@@ -1,6 +1,6 @@
 /* scp.h: simulator control program headers
 
-   Copyright (c) 1993-2005, Robert M Supnik
+   Copyright (c) 1993-2006, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -23,29 +23,32 @@
    be used in advertising or otherwise to promote the sale, use or other dealings
    in this Software without prior written authorization from Robert M Supnik.
 
-   07-Feb-05	RMS	Added ASSERT command
-   09-Sep-04	RMS	Added reset_all_p
-   14-Feb-04	RMS	Added debug prototypes (from Dave Hittner)
-   02-Jan-04	RMS	Split out from SCP
+   14-Jul-06    RMS     Added sim_activate_abs
+   06-Jan-06    RMS     Added fprint_stopped_gen
+                        Changed arg type in sim_brk_test
+   07-Feb-05    RMS     Added ASSERT command
+   09-Sep-04    RMS     Added reset_all_p
+   14-Feb-04    RMS     Added debug prototypes (from Dave Hittner)
+   02-Jan-04    RMS     Split out from SCP
 */
 
 #ifndef _SIM_SCP_H_
-#define _SIM_SCP_H_	0
+#define _SIM_SCP_H_     0
 
 /* run_cmd parameters */
 
-#define RU_RUN		0				/* run */
-#define RU_GO		1				/* go */
-#define RU_STEP		2				/* step */
-#define RU_CONT		3				/* continue */
-#define RU_BOOT		4				/* boot */
+#define RU_RUN          0                               /* run */
+#define RU_GO           1                               /* go */
+#define RU_STEP         2                               /* step */
+#define RU_CONT         3                               /* continue */
+#define RU_BOOT         4                               /* boot */
 
 /* get_sim_opt parameters */
 
-#define CMD_OPT_SW	001				/* switches */
-#define CMD_OPT_OF	002				/* output file */
-#define CMD_OPT_SCH	004				/* search */
-#define CMD_OPT_DFT	010				/* defaults */
+#define CMD_OPT_SW      001                             /* switches */
+#define CMD_OPT_OF      002                             /* output file */
+#define CMD_OPT_SCH     004                             /* search */
+#define CMD_OPT_DFT     010                             /* defaults */
 
 /* Command processors */
 
@@ -74,6 +77,7 @@ t_stat echo_cmd (int32 flag, char *ptr);
 
 t_stat sim_process_event (void);
 t_stat sim_activate (UNIT *uptr, int32 interval);
+t_stat sim_activate_abs (UNIT *uptr, int32 interval);
 t_stat sim_cancel (UNIT *uptr);
 int32 sim_is_active (UNIT *uptr);
 double sim_gtime (void);
@@ -90,7 +94,7 @@ char *get_glyph (char *iptr, char *optr, char mchar);
 char *get_glyph_nc (char *iptr, char *optr, char mchar);
 t_value get_uint (char *cptr, uint32 radix, t_value max, t_stat *status);
 char *get_range (DEVICE *dptr, char *cptr, t_addr *lo, t_addr *hi,
-	uint32 rdx, t_addr max, char term);
+    uint32 rdx, t_addr max, char term);
 t_stat get_ipaddr (char *cptr, uint32 *ipa, uint32 *ipp);
 t_value strtotv (char *cptr, char **endptr, uint32 radix);
 t_stat fprint_val (FILE *stream, t_value val, uint32 rdx, uint32 wid, uint32 fmt);
@@ -103,11 +107,13 @@ CTAB *find_ctab (CTAB *tab, char *gbuf);
 C1TAB *find_c1tab (C1TAB *tab, char *gbuf);
 SHTAB *find_shtab (SHTAB *tab, char *gbuf);
 BRKTAB *sim_brk_fnd (t_addr loc);
-t_bool sim_brk_test (t_addr bloc, int32 btyp);
+uint32 sim_brk_test (t_addr bloc, uint32 btyp);
+void sim_brk_clrspc (uint32 spc);
 char *match_ext (char *fnam, char *ext);
 t_stat sim_cancel_step (void);
 void sim_debug_u16 (uint32 dbits, DEVICE* dptr, const char* const* bitdefs,
-	uint16 before, uint16 after, int terminate);
+    uint16 before, uint16 after, int terminate);
 void sim_debug (uint32 dbits, DEVICE* dptr, const char* fmt, ...);
+void fprint_stopped_gen (FILE *st, t_stat v, REG *pc, DEVICE *dptr);
 
 #endif
