@@ -1,6 +1,6 @@
 /* sim_console.c: simulator console I/O library
 
-   Copyright (c) 1993-2005, Robert M Supnik
+   Copyright (c) 1993-2006, Robert M Supnik
 
    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
@@ -23,6 +23,9 @@
    used in advertising or otherwise to promote the sale, use or other dealings
    in this Software without prior written authorization from Robert M Supnik.
 
+   22-Jun-06    RMS     Implemented SET/SHOW PCHAR
+   31-May-06    JDB     Fixed bug if SET CONSOLE DEBUG with no argument
+   22-Nov-05    RMS     Added central input/output conversion support
    05-Nov-04    RMS     Moved SET/SHOW DEBUG under CONSOLE hierarchy
    28-Oct-04    JDB     Fixed SET CONSOLE to allow comma-separated parameters
    20-Aug-04    RMS     Added OS/2 EMX fixes (from Holger Veit)
@@ -58,6 +61,8 @@
    sim_putchar_s -      output character to console, stall if congested
    sim_set_console -    set console parameters
    sim_show_console -   show console parameters
+   sim_tt_inpcvt -      convert input character per mode
+   sim_tt_outcvt -      convert output character per mode
 
    sim_ttinit   -       called once to get initial terminal state
    sim_ttrun    -       called to put terminal into run state
@@ -78,6 +83,7 @@
 #include "sim_defs.h"
 #include "sim_sock.h"
 #include "sim_tmxr.h"
+#include <ctype.h>
 
 #define KMAP_WRU        0
 #define KMAP_BRK        1
@@ -87,6 +93,7 @@
 
 int32 sim_int_char = 005;                               /* interrupt character */
 int32 sim_brk_char = 000;                               /* break character */
+int32 sim_tt_pchar = 0x00002780;
 #if defined (_WIN32) || defined (__OS2__) || (defined (__MWERKS__) && defined (macintosh))
 int32 sim_del_char = '\b';                              /* delete character */
 #else
@@ -106,6 +113,7 @@ static CTAB set_con_tab[] = {
     { "WRU", &sim_set_kmap, KMAP_WRU | KMAP_NZ },
     { "BRK", &sim_set_kmap, KMAP_BRK },
     { "DEL", &sim_set_kmap, KMAP_DEL |KMAP_NZ },
+    { "PCHAR", &sim_set_pchar, 0 },
     { "TELNET", &sim_set_telnet, 0 },
     { "NOTELNET", &sim_set_notelnet, 0 },
     { "LOG", &sim_set_logon, 0 },
@@ -119,6 +127,7 @@ static SHTAB show_con_tab[] = {
     { "WRU", &sim_show_kmap, KMAP_WRU },
     { "BRK", &sim_show_kmap, KMAP_BRK },
     { "DEL", &sim_show_kmap, KMAP_DEL },
+    { "PCHAR", &sim_show_pchar, 0 },
     { "LOG", &sim_show_log, 0 },
     { "TELNET", &sim_show_telnet, 0 },
     { "DEBUG", &sim_show_debug, 0 },
@@ -205,13 +214,37 @@ return SCPE_OK;
 
 t_stat sim_show_kmap (FILE *st, DEVICE *dptr, UNIT *uptr, int32 flag, char *cptr)
 {
-int32 rdx = 8;
+if (sim_devices[0]->dradix == 16)
+    fprintf (st, "%s = %X\n", show_con_tab[flag].name, *(cons_kmap[flag & KMAP_MASK]));
+else fprintf (st, "%s = %o\n", show_con_tab[flag].name, *(cons_kmap[flag & KMAP_MASK]));
+return SCPE_OK;
+}
 
-dptr = sim_devices[0];
+/* Set printable characters */
+
+t_stat sim_set_pchar (int32 flag, char *cptr)
+{
+DEVICE *dptr = sim_devices[0];
+uint32 val, rdx;
+t_stat r;
+
+if ((cptr == NULL) || (*cptr == 0)) return SCPE_2FARG;
 if (dptr->dradix == 16) rdx = 16;
-fprintf (st, "%s = ", show_con_tab[flag].name);
-fprint_val (st, *(cons_kmap[flag & KMAP_MASK]), rdx, 7, 0);
-fprintf (st, "\n");
+else rdx = 8;
+val = (uint32) get_uint (cptr, rdx, 0xFFFFFFFF, &r);
+if ((r != SCPE_OK) ||
+    ((val & 0x00002400) == 0)) return SCPE_ARG;
+sim_tt_pchar = val;
+return SCPE_OK;
+}
+
+/* Show printable characters */
+
+t_stat sim_show_pchar (FILE *st, DEVICE *dptr, UNIT *uptr, int32 flag, char *cptr)
+{
+if (sim_devices[0]->dradix == 16)
+    fprintf (st, "pchar mask = %X\n", sim_tt_pchar);
+else fprintf (st, "pchar mask = %o\n", sim_tt_pchar);
 return SCPE_OK;
 }
 
@@ -261,7 +294,7 @@ t_stat sim_set_debon (int32 flag, char *cptr)
 {
 char *tptr, gbuf[CBUFSIZE];
 
-if (*cptr == 0) return SCPE_2FARG;                      /* need arg */
+if ((cptr == NULL) || (*cptr == 0)) return SCPE_2FARG;  /* too few arguments? */
 tptr = get_glyph (cptr, gbuf, 0);                       /* get file name */
 if (*tptr != 0) return SCPE_2MARG;                      /* now eol? */
 sim_set_deboff (0, NULL);                               /* close cur debug */
@@ -416,6 +449,41 @@ tmxr_poll_tx (&sim_con_tmxr);                           /* poll xmt */
 return r;                                               /* return status */
 }
 
+/* Input character processing */
+
+int32 sim_tt_inpcvt (int32 c, uint32 mode)
+{
+uint32 md = mode & TTUF_M_MODE;
+
+if (md != TTUF_MODE_8B) {
+    c = c & 0177;
+    if (md == TTUF_MODE_UC) {
+        if (islower (c)) c = toupper (c);
+        if (mode & TTUF_KSR) c = c | 0200;
+        }
+    }
+else c = c & 0377;
+return c;
+}
+
+/* Output character processing */
+
+int32 sim_tt_outcvt (int32 c, uint32 mode)
+{
+uint32 md = mode & TTUF_M_MODE;
+
+if (md != TTUF_MODE_8B) {
+    c = c & 0177;
+    if ((md == TTUF_MODE_UC) && islower (c)) c = toupper (c);
+    if (((md == TTUF_MODE_UC) || (md == TTUF_MODE_7P)) &&
+        ((c == 0177) ||
+         ((c < 040) && !((sim_tt_pchar >> c) & 1))))
+        return -1;
+    }
+else c = c & 0377;
+return c;
+}
+
 /* VMS routines, from Ben Thomas, with fixes from Robert Alan Byer */
 
 #if defined (VMS)
